@@ -31,7 +31,6 @@ import univention.admin.objects
 import univention.admin.uldap
 import univention.debug as ud
 import univention.logging
-import univention.uldap
 from univention.logging import Structured
 from univention.s4connector.lockingdb import LockingDB
 from univention.s4connector.s4cache import S4Cache
@@ -82,11 +81,9 @@ def set_primary_group_user(connector, key, ucs_object):
     """check if correct primary group is set"""
     connector.set_primary_group_to_ucs_user(key, ucs_object)
 
+
 # compare functions
-
 # helper
-
-
 def dictonary_lowercase(dict_):
     if isinstance(dict_, dict):
         ndict = {}
@@ -117,6 +114,7 @@ def compare_lowercase(val1, val2):
     except Exception:  # FIXME: which exception is to be caught?
         return False
 
+
 # helper classes
 
 
@@ -279,48 +277,52 @@ class attribute:
     """
     A mapping attribute description
 
-    :param ucs_attribute:
-    The property name of the object in UDM
+    :param ucs_attribute: The property name of the object in UDM
     :type ucs_attribute: str
 
-    :param ldap_attribute:
-    The LDAP attribute name of the object in UCS LDAP
+    :param ldap_attribute: The LDAP attribute name of the object in UCS LDAP
     :type ldap_attribute: str
 
-    :param con_attribute:
-    The LDAP attribute name of the object in AD LDAP
+    :param con_attribute: The LDAP attribute name of the object in AD LDAP
     :type con_attribute: str
 
-    :param con_other_attribute:
-    Further LDAP attribute name of the object in AD LDAP.
+    :param con_other_attribute: Further LDAP attribute name of the object in AD LDAP.
     :type con_other_attribute: str
 
-    :param required:
-    unused
+    :param required: unused
     :type required: bool
 
-    :param single_value:
-    Whether the attribute is single_value in the AD LDAP.
+    :param single_value: Whether the attribute is single_value in the AD LDAP.
     :type single_value: bool
 
-    :param compare_function:
-    A comparison function which compares raw ldap attribute values.
+    :param compare_function: A comparison function which compares raw ldap attribute values.
     :type compare_function: callable
 
-    :param mapping:
-    Mapping functions for (sync_to_s4, sync_to_ucs)
+    :param mapping: Mapping functions for (sync_to_s4, sync_to_ucs)
     :ptype mapping: tuple
 
-    :param reverse_attribute_check:
-    Make a reverse check of this mapping, if the mapping is not 1:1.
+    :param reverse_attribute_check: Make a reverse check of this mapping, if the mapping is not 1:1.
     :ptype reverse_attribute_check: bool
 
-    :param sync_mode:
-    The syncronization direction (read, write, sync)
+    :param sync_mode: The syncronization direction (read, write, sync)
     :ptype sync_mode: str
     """
 
-    def __init__(self, ucs_attribute='', ldap_attribute='', con_attribute='', con_other_attribute='', required=0, single_value=False, compare_function=None, mapping=(), reverse_attribute_check=False, sync_mode='sync', con_attribute_encoding='UTF-8', auto_enable_udm_option=False):
+    def __init__(
+        self,
+        ucs_attribute='',
+        ldap_attribute='',
+        con_attribute='',
+        con_other_attribute='',
+        required=0,
+        single_value=False,
+        compare_function=None,
+        mapping=(),
+        reverse_attribute_check=False,
+        sync_mode='sync',
+        con_attribute_encoding='UTF-8',
+        auto_enable_udm_option=False,
+    ):
         self.ucs_attribute = ucs_attribute
         self.ldap_attribute = ldap_attribute
         self.con_attribute = con_attribute
@@ -361,35 +363,35 @@ class attribute:
 class property:
 
     def __init__(
-            self,
-            ucs_default_dn='',
-            con_default_dn='',
-            ucs_module='',
-            ucs_module_others=[],
-            sync_mode='',
-            scope='',
-            con_search_filter='',
-            ignore_filter=None,
-            match_filter=None,
-            ignore_subtree=[],
-            con_create_objectclass=[],
-            con_create_attributes=[],
-            dn_mapping_function=[],
-            attributes=None,
-            ucs_create_functions=[],
-            con_create_extensions=[],
-            post_con_create_functions=[],
-            post_con_modify_functions=[],
-            post_ucs_modify_functions=[],
-            post_attributes=None,
-            mapping_table=None,
-            position_mapping=[],
-            con_sync_function=None,
-            ucs_sync_function=None,
-            disable_delete_in_ucs=False,
-            identify=None,
-            con_subtree_delete_objects=[]):
-
+        self,
+        ucs_default_dn='',
+        con_default_dn='',
+        ucs_module='',
+        ucs_module_others=[],
+        sync_mode='',
+        scope='',
+        con_search_filter='',
+        ignore_filter=None,
+        match_filter=None,
+        ignore_subtree=[],
+        con_create_objectclass=[],
+        con_create_attributes=[],
+        dn_mapping_function=[],
+        attributes=None,
+        ucs_create_functions=[],
+        con_create_extensions=[],
+        post_con_create_functions=[],
+        post_con_modify_functions=[],
+        post_ucs_modify_functions=[],
+        post_attributes=None,
+        mapping_table=None,
+        position_mapping=[],
+        con_sync_function=None,
+        ucs_sync_function=None,
+        disable_delete_in_ucs=False,
+        identify=None,
+        con_subtree_delete_objects=[],
+    ):
         self.ucs_default_dn = ucs_default_dn
 
         self.con_default_dn = con_default_dn
@@ -465,7 +467,6 @@ class property:
 class ucs:
 
     def __init__(self, CONFIGBASENAME, _property, configRegistry, listener_dir, logfilename, debug_level):
-
         self.CONFIGBASENAME = CONFIGBASENAME
 
         self.configRegistry = configRegistry
@@ -805,7 +806,6 @@ class ucs:
             return True
 
     def get_ucs_ldap_object_dn(self, dn):
-
         for _i in [0, 1]:  # do it twice if the LDAP connection was closed
             try:
                 return self.lo.lo.lo.search_s(dn, ldap.SCOPE_BASE, '(objectClass=*)', ('dn',))[0][0]
@@ -820,7 +820,6 @@ class ucs:
                 continue
 
     def get_ucs_ldap_object(self, dn):
-
         for _i in [0, 1]:  # do it twice if the LDAP connection was closed
             try:
                 return self.lo.get(dn, required=1)
@@ -1099,7 +1098,11 @@ class ucs:
             con_attribute = attributes.con_attribute
             con_other_attribute = attributes.con_other_attribute
 
-            if not object.get('changed_attributes') or con_attribute in object.get('changed_attributes') or (con_other_attribute and con_other_attribute in object.get('changed_attributes')):
+            if (
+                not object.get('changed_attributes')
+                or con_attribute in object.get('changed_attributes')
+                or (con_other_attribute and con_other_attribute in object.get('changed_attributes'))
+            ):
                 log.trace("__set_values: Set: %s", con_attribute)
                 set_values(attributes)
             else:
@@ -1117,7 +1120,11 @@ class ucs:
             con_attribute = post_attributes.con_attribute
             con_other_attribute = post_attributes.con_other_attribute
 
-            if not object.get('changed_attributes') or con_attribute in object.get('changed_attributes') or (con_other_attribute and con_other_attribute in object.get('changed_attributes')):
+            if (
+                not object.get('changed_attributes')
+                or con_attribute in object.get('changed_attributes')
+                or (con_other_attribute and con_other_attribute in object.get('changed_attributes'))
+            ):
                 log.trace("__set_values: Set: %s", con_attribute)
                 if post_attributes.reverse_attribute_check:
                     if object['attributes'].get(post_attributes.ldap_attribute):
@@ -1168,7 +1175,6 @@ class ucs:
         return res
 
     def modify_in_ucs(self, property_type, object, module, position):
-
         # reload extended attributes  # FIXME: maybe not necessary
         univention.admin.modules.init(self.lo, univention.admin.uldap.position(self.lo.base), module)
 
@@ -1572,9 +1578,7 @@ class ucs:
                 return False
 
         def connecting_filter(filter, attributes):
-
             def walk(filter, attributes):
-
                 def split(filter):
                     opened = []
                     closed = []
@@ -1618,7 +1622,6 @@ class ucs:
                 return 0 not in walk(filter[1:], attributes)
 
         def subfilter(filter, attributes):
-
             if filter[0] == '(':
                 if not filter[-1] == ')':
                     raise ValueError(f"matching ) missing in filter: {filter}")
