@@ -9,6 +9,7 @@
 
 import binascii
 import time
+from datetime import datetime
 from logging import getLogger
 
 import heimdal
@@ -18,6 +19,7 @@ from samba.dcerpc import drsblobs
 from samba.ndr import ndr_pack, ndr_print, ndr_unpack
 
 import univention.s4connector.s4
+from univention.admin import uexceptions
 from univention.logging import Structured
 from univention.s4connector.s4 import format_escaped
 
@@ -433,7 +435,6 @@ def calculate_supplementalCredentials(ucs_krb5key, old_supplementalCredentials,
 
 
 def extract_NThash_from_krb5key(ucs_krb5key):
-
     NThash = None
 
     for k in ucs_krb5key:
@@ -815,7 +816,7 @@ def password_sync_s4_to_ucs(s4connector, key, ucs_object, modifyUserPassword=Tru
                 # we need to expire the password. Since shadowMax=0 is its minimum value, we need to set shadowLastChange = today-2days ## FIXME: -1day should be enough
                 two_days_ago = int(time.time()) - 86400 * 2
                 new_shadowLastChange = str(two_days_ago // 3600 // 24).encode('ASCII')
-            else:                # not pwd change on next login
+            else:  # not pwd change on next login
                 new_shadowLastChange = str(pwdLastSet_unix // 3600 // 24).encode('ASCII')
                 userobject = s4connector.get_ucs_object(key, ucs_object['dn'])
                 if not userobject:
@@ -873,163 +874,172 @@ def password_sync_s4_to_ucs_no_userpassword(s4connector, key, ucs_object):
     password_sync_s4_to_ucs(s4connector, key, ucs_object, modifyUserPassword=False)
 
 
-def lockout_sync_s4_to_ucs(s4connector, key, ucs_object):
+def lockout_sync_to_ucs(connector, key, obj):
     """
-    Sync account locking *state* from Samba/AD to OpenLDAP:
-            sync Samba/AD (lockoutTime != 0)      ->  OpenLDAP sambaAcctFlags ("L")
-            and  Samba/AD badPasswordTime         ->  OpenLDAP sambaBadPasswordTime
+    Sync account locking *state* from AD to UCS:
+        sync AD (lockoutTime != 0)      ->  UCS locked = 1 and lockedTime = lockoutTime
+                (lockoutTime == 0)      ->  UCS locked = 0 and lockedTime = 0 (lockedTime would be set automaticly by UCS)
     """
-    function_name = 'lockout_sync_s4_to_ucs'
-    log.debug("%s called", function_name)
-
-    if ucs_object['modtype'] not in ('modify', 'add'):
+    if obj['modtype'] not in ('modify', 'add'):
         return
 
-    modlist = []
+    if 'lockoutTime' not in obj['changed_attributes']:
+        log.trace(
+            'No lockout attribute in changed attributes.',
+            changed_attributes=obj['changed_attributes'],
+            dn=obj['dn'],
+        )
+        return
 
     try:
-        ucs_object_attributes = s4connector.lo.get(ucs_object['dn'], ['sambaAcctFlags', 'sambaBadPasswordTime'], required=True)
+        udm_object = univention.admin.objects.get(
+            connector.modules[key],
+            co=None,
+            lo=connector.lo,
+            position='',
+            dn=obj['dn'],
+        )
+    except uexceptions.noObject:
+        log.warning('Object with DN %s not found!', obj['dn'])
+        return
+
+    udm_object.open()
+
+    is_locked = udm_object['locked'] == '1'
+    lockout_time = obj['attributes'].get('lockoutTime', [b'0'])[0]
+
+    # Convert AD lockoutTime (Windows Filetime) to UCS lockedTime (GeneralizedTime)
+    lockout_time_unix_ts = int(lockout_time.decode('ascii')) / 10000000 - 11644473600
+    locked_time = datetime.fromtimestamp(lockout_time_unix_ts).strftime('%Y%m%d%H%M%SZ')
+
+    should_be_locked = lockout_time != b'0'
+    log.debug(
+        'Locking user account',
+        locked_time=locked_time,
+        lockout_time=lockout_time,
+        is_locked=is_locked,
+        should_be_locked=should_be_locked,
+        dn=obj['dn'],
+    )
+
+    if should_be_locked == is_locked:
+        return
+
+    old_states = (
+        udm_object.descriptions['locked'].editable,
+        udm_object.descriptions['locked'].may_change,
+        udm_object.descriptions['lockedTime'].editable,
+        udm_object.descriptions['lockedTime'].may_change,
+    )
+    (
+        udm_object.descriptions['locked'].editable,
+        udm_object.descriptions['locked'].may_change,
+        udm_object.descriptions['lockedTime'].editable,
+        udm_object.descriptions['lockedTime'].may_change,
+    ) = (True, True, True, True)
+
+    try:
+        if should_be_locked:
+            log.process('Lock user in UCS.', dn=obj['dn'])
+            udm_object['locked'] = '1'
+            udm_object['lockedTime'] = locked_time
+        else:
+            log.process('Unlock user in UCS.', dn=obj['dn'])
+            udm_object['locked'] = '0'
+
+        udm_object.modify()
+    finally:
+        (
+            udm_object.descriptions['locked'].editable,
+            udm_object.descriptions['locked'].may_change,
+            udm_object.descriptions['lockedTime'].editable,
+            udm_object.descriptions['lockedTime'].may_change,
+        ) = old_states
+
+
+def lockout_sync_from_ucs(connector, key, obj):
+    """
+    Sync unlock *modification* from OpenLDAP to AD:
+        sync OpenLDAP ("L" not in sambaAcctFlags) -> AD lockoutTime = 0
+
+        sync OpenLDAP ("L" in sambaAcctFlags)     -> AD lockoutTime = sambaBadPasswordTime
+        and  OpenLDAP sambaBadPasswordTime        -> AD badPasswordTime
+    """
+    if obj['modtype'] not in ('modify', 'add'):
+        return
+
+    if obj.get('new_ucs_object') and obj.get('old_ucs_object'):
+        is_locked = b'L' in obj['new_ucs_object'].get('sambaAcctFlags', [b''])[0]
+        was_locked = b'L' in obj['old_ucs_object'].get('sambaAcctFlags', [b''])[0]
+        if is_locked == was_locked:
+            return
+
+    ucs_object = connector._object_mapping(key, obj, 'con')
+
+    try:
+        ucs_object_attributes = connector.lo.get(ucs_object['dn'], ['sambaAcctFlags', 'sambaBadPasswordTime'], required=True)
     except ldap.NO_SUCH_OBJECT:
-        log.warning("%s: The UCS object (%s) was not found. The object was removed.", function_name, ucs_object['dn'])
-        return
-    sambaAcctFlags = ucs_object_attributes.get('sambaAcctFlags', [b''])[0]
-    sambaBadPasswordTime = ucs_object_attributes.get('sambaBadPasswordTime', [b"0"])[0]
-
-    lockoutTime = ucs_object['attributes'].get('lockoutTime', [b'0'])[0]
-    if lockoutTime != b"0":
-        if b"L" not in sambaAcctFlags:
-            acctFlags = univention.admin.samba.acctFlags(sambaAcctFlags)
-            new_sambaAcctFlags = acctFlags.set('L').encode('ASCII')
-            log.process("%s: Marking Samba account as locked in OpenLDAP", function_name)
-            modlist.append(('sambaAcctFlags', sambaAcctFlags, new_sambaAcctFlags))
-
-        badPasswordTime = ucs_object['attributes'].get('badPasswordTime', [b"0"])[0]
-        if badPasswordTime != sambaBadPasswordTime:
-            log.process("%s: Copying badPasswordTime from S4: %s", function_name, badPasswordTime)
-            if sambaBadPasswordTime:
-                log.debug("%s: Old sambaBadPasswordTime: %s", function_name, sambaBadPasswordTime)
-            modlist.append(('sambaBadPasswordTime', sambaBadPasswordTime, badPasswordTime))
-    else:
-        if b"L" in sambaAcctFlags:
-            acctFlags = univention.admin.samba.acctFlags(sambaAcctFlags)
-            new_sambaAcctFlags = acctFlags.unset('L').encode('ASCII')
-            log.process("%s: Marking Samba account as unlocked in OpenLDAP", function_name)
-            modlist.append(('sambaAcctFlags', sambaAcctFlags, new_sambaAcctFlags))
-
-        if sambaBadPasswordTime and sambaBadPasswordTime != b"0":
-            log.process("%s: Unsetting sambaBadPasswordTime: %s", function_name, sambaBadPasswordTime)
-            modlist.append(('sambaBadPasswordTime', sambaBadPasswordTime, b"0"))
-
-    if modlist:
-        log.trace("%s: modlist: %s", function_name, modlist)
-        s4connector.lo.lo.modify(ucs_object['dn'], modlist)
-
-
-def lockout_sync_ucs_to_s4(s4connector, key, object):
-    """
-    Sync unlock *modification* from OpenLDAP to Samba/AD:
-            sync OpenLDAP ("L" not in sambaAcctFlags) ->  Samba/AD lockoutTime = 0
-
-            sync OpenLDAP ("L" in sambaAcctFlags) ->  Samba/AD lockoutTime = sambaBadPasswordTime
-            and  OpenLDAP sambaBadPasswordTime    ->  Samba/AD badPasswordTime
-    """
-    function_name = 'lockout_sync_ucs_to_s4'
-    log.debug("%s called", function_name)
-
-    if object['modtype'] not in ('modify', 'add'):
+        log.warning('The UCS object (%s) was not found. The object was removed.', ucs_object['dn'])
         return
 
-    new_ucs_object = object.get('new_ucs_object', {})
-    if not new_ucs_object:
-        # only set by sync_from_ucs in MODIFY case
+    try:
+        ad_object_attributes = connector.lo_s4.get(obj['dn'], ['lockoutTime'], required=True)
+    except ldap.NO_SUCH_OBJECT:
+        log.warning('The AD object (%s) was not found. The object was removed.', obj['dn'])
         return
 
-    old_ucs_object = object.get('old_ucs_object', {})
-    if not old_ucs_object:
-        # only set by sync_from_ucs in MODIFY case
-        return
+    samba_acct_flags = ucs_object_attributes.get('sambaAcctFlags', [b''])[0]
+    is_locked = b'L' in samba_acct_flags
 
-    new_sambaAcctFlags = new_ucs_object.get('sambaAcctFlags', [b''])[0]
-    is_locked = b"L" in new_sambaAcctFlags
+    samba_bad_password_time = ucs_object_attributes.get('sambaBadPasswordTime', [b'0'])[0]
+    lockout_time = ad_object_attributes.get('lockoutTime', [b'0'])[0]
+    should_be_locked = lockout_time != b'0'
 
-    old_sambaAcctFlags = old_ucs_object.get('sambaAcctFlags', [b''])[0]
-    was_locked = b"L" in old_sambaAcctFlags
+    log.debug(
+        'Lockout states.',
+        samba_acct_flags=samba_acct_flags,
+        samba_bad_password_time=samba_bad_password_time,
+        lockout_time=lockout_time,
+        is_locked=is_locked,
+        should_be_locked=should_be_locked,
+        ucs_object_dn=ucs_object['dn'],
+        ad_object_dn=obj['dn'],
+    )
 
-    if is_locked == was_locked:
-        # Require a change in the pickled state
+    if should_be_locked == is_locked:
         return
 
     modlist = []
     if not is_locked:
-        s4_object_attributes = s4connector.lo_s4.get(object['dn'], ['lockoutTime', 'badPasswordTime'])
-        if 'lockoutTime' not in s4_object_attributes:
-            return
-
-        lockoutTime = s4_object_attributes['lockoutTime'][0]
-        if lockoutTime == b"0":
-            return
-
-        # Now object.get('new_ucs_object') may be a stale pickled state, so let's lookup the current OpenLDAP object state
-        # Unfortunately "object" doesn't hold the current OpenLDAP DN, so we need to map back first
-        ucs_object = s4connector._object_mapping(key, object)
-        try:
-            ucs_object_attributes = s4connector.lo.get(ucs_object['dn'], ['sambaAcctFlags', 'sambaBadPasswordTime'], required=True)
-        except ldap.NO_SUCH_OBJECT:
-            log.warning("%s: The UCS object (%s) was not found. The object was removed.", function_name, ucs_object['dn'])
-            return
-        sambaAcctFlags = ucs_object_attributes.get('sambaAcctFlags', [b''])[0]
-
-        if b"L" in sambaAcctFlags:
-            # currently locked again
-            return
-
-        sambaBadPasswordTime = ucs_object_attributes.get('sambaBadPasswordTime', [b''])[0]
-        if sambaBadPasswordTime and sambaBadPasswordTime != b"0":
-            log.error("%s: The UCS object (%s) is unlocked, but sambaBadPasswordTime is set.", function_name, ucs_object['dn'])
+        if samba_bad_password_time and samba_bad_password_time != b'0':
+            log.error('The UCS object (%s) is unlocked, but sambaBadPasswordTime is set.', ucs_object['dn'])
             return
 
         # Ok here we have:
-        # 1. Account currently not locked in OpenLDAP but in Samba/AD
+        # 1. Account currently not locked in OpenLDAP but in AD
         # 2. Lockout state has changed to unlocked at some pickled point in the past
-        modlist.append((ldap.MOD_REPLACE, "lockoutTime", b"0"))
-        modlist.append((ldap.MOD_REPLACE, "badPasswordTime", b"0"))
-        log.process("%s: Marking account as unlocked in Samba/AD", function_name)
+        modlist.append((ldap.MOD_REPLACE, 'lockoutTime', b'0'))
+        modlist.append((ldap.MOD_REPLACE, 'badPasswordTime', b'0'))
+        log.process('Unlock user in AD.', dn=obj['dn'])
     else:
-        s4_object_attributes = s4connector.lo_s4.get(object['dn'], ['lockoutTime', 'badPasswordTime'])
-        lockoutTime = s4_object_attributes.get('lockoutTime', [b'0'])[0]
-
-        # Now object.get('new_ucs_object') may be a stale pickled state, so let's lookup the current OpenLDAP object state
-        # Unfortunately "object" doesn't hold the current OpenLDAP DN, so we need to map back first
-        ucs_object = s4connector._object_mapping(key, object)
-        try:
-            ucs_object_attributes = s4connector.lo.get(ucs_object['dn'], ['sambaAcctFlags', 'sambaBadPasswordTime'], required=True)
-        except ldap.NO_SUCH_OBJECT:
-            log.warning("%s: The UCS object (%s) was not found. The object was removed.", function_name, ucs_object['dn'])
+        if not samba_bad_password_time:
+            log.error('The UCS object (%s) is locked, but sambaBadPasswordTime is missing.', ucs_object['dn'])
             return
-        sambaAcctFlags = ucs_object_attributes.get('sambaAcctFlags', [b''])[0]
-        if b"L" not in sambaAcctFlags:
-            # currently not locked any longer
+        if samba_bad_password_time == b'0':
+            log.error('The UCS object (%s) is locked, but sambaBadPasswordTime is 0.', ucs_object['dn'])
             return
-
-        sambaBadPasswordTime = ucs_object_attributes.get('sambaBadPasswordTime', [b''])[0]
-        if not sambaBadPasswordTime:
-            log.error("%s: The UCS object (%s) is locked, but sambaBadPasswordTime is missing.", function_name, ucs_object['dn'])
-            return
-        if sambaBadPasswordTime == b"0":
-            log.error("%s: The UCS object (%s) is locked, but sambaBadPasswordTime is 0.", function_name, ucs_object['dn'])
-            return
-        if sambaBadPasswordTime == lockoutTime:
+        if samba_bad_password_time == lockout_time:
             # already locked
             return
 
         # Ok here we have:
-        # 1. Account currently locked in OpenLDAP but not in Samba/AD
+        # 1. Account currently locked in OpenLDAP but not in AD
         # 2. Lockout state has changed to locked at some pickled point in the past
-        modlist.append((ldap.MOD_REPLACE, "lockoutTime", sambaBadPasswordTime))
-        modlist.append((ldap.MOD_REPLACE, "badPasswordTime", sambaBadPasswordTime))
-        log.process("%s: Marking account as locked in Samba/AD", function_name)
-        log.debug("%s: Setting lockoutTime to the value of sambaBadPasswordTime: %s", function_name, sambaBadPasswordTime)
+        modlist.append((ldap.MOD_REPLACE, 'lockoutTime', samba_bad_password_time))
+        modlist.append((ldap.MOD_REPLACE, 'badPasswordTime', samba_bad_password_time))
+        log.process('Lock user in AD.', dn=obj['dn'])
+        log.debug('Setting lockoutTime to the value of sambaBadPasswordTime: %s', samba_bad_password_time)
 
     if modlist:
-        log.trace("%s: modlist: %s", function_name, modlist)
-        s4connector.lo_s4.lo.modify_ext_s(object['dn'], modlist)
+        log.trace('modlist: %s', modlist)
+        connector.lo_s4.lo.modify_ext_s(obj['dn'], modlist)
